package org.slf4j;

import java.util.logging.Logger;
import java.util.logging.Handler;

/**
 * The system property, or Preferences API, should be used to set the 
 * "java.util.logging.config.class" property to the value
 * "org.slf4j.SLF4JConfig"
 */
public class SLF4JConfig {

  /**
   * The default constructor will be invoked by the LogManager on
   * demand in order to load the configuration. This particular object
   * will "load" a configuration by installing the SLF4JHandler as the
   * root Handler if it is not installed already.
   */
  public SLF4JConfig() {
    installHandlers();
  }

  /**
   * Install the JUL Handler if its not already installed
   */
  protected void installHandlers() {

    Logger log = Logger.getLogger("");
    boolean installed = false;

    synchronized(SLF4JConfig.class) {

      // Check the current Handler on the root Logger.
      Handler[] handler = log.getHandlers();
      for(int i = 0; handler != null && i < handler.length; i++) {
        if(handler[i] != null && handler[i].getClass().equals( SLF4JHandler.class ))
          installed = true;
      }

      // If the SLF4JHandler is not one of them, then install it.
      if(!installed)
        log.addHandler( new SLF4JHandler() );

    }

  }

}

