package org.slf4j;

import java.util.logging.ErrorManager;
import java.util.logging.Handler;
import java.util.logging.LogManager;
import java.util.logging.LogRecord;
import java.util.logging.Level;
import java.util.logging.SimpleFormatter;
import org.slf4j.LoggerFactory;
import org.slf4j.Logger;

/**
 */
public class SLF4JHandler extends Handler {

  public SLF4JHandler() {
    configure();
  }

  /**
   * Configure the Handler to intercept everything and hand it off to SLF4J
   */ 
  protected void configure() {

    setLevel(Level.FINEST);
    setFilter(null);
    setFormatter(new SimpleFormatter());

    try {
      setEncoding(null);
    } catch (Exception shouldNotOccurr) {
    }

  }

  public void close()
    throws SecurityException {}

  public void flush()
    throws SecurityException {}

  /**
   * Delegate the logging to SLF4J
   */
  public void publish(LogRecord record) {

    if(record == null)
      return;

    String name = record.getLoggerName();
    Level level = record.getLevel();

    if(Level.OFF == level)
      return;

    // Format the message
    String message;
    try {
      message = getFormatter().format(record);
    } catch (Exception ex) {
      reportError(null, ex, ErrorManager.FORMAT_FAILURE);
      return;
    }

    // Finally send it to the correct logger
    Logger logger = LoggerFactory.getLogger(name);
    Throwable cause = record.getThrown();

    if(cause == null) {

      if(Level.FINE == level || Level.FINER == level || Level.FINEST == level)
        logger.debug(message);
      else if(Level.ALL == level || Level.INFO == level)
        logger.info(message);
      else if(Level.WARNING == level)
        logger.warn(message);
      else if(Level.SEVERE == level)
        logger.error(message);

    } else {

      if(Level.FINE == level || Level.FINER == level || Level.FINEST == level)
        logger.debug(message);
      else if(Level.ALL == level || Level.INFO == level)
        logger.info(message);
      else if(Level.WARNING == level)
        logger.warn(message);
      else if(Level.SEVERE == level)
        logger.error(message);

    }

  }

  /**
   * Delegate the isLoggable() test to SLF4J
   */
  public boolean isLoggable(LogRecord record) {
  
    if(record == null)
      return false;

    String name = record.getLoggerName();
    Level level = record.getLevel();

    if(Level.OFF == level)
      return false;

    Logger logger = LoggerFactory.getLogger(name);
    if(Level.FINE == level || Level.FINER == level || Level.FINEST == level)
      return logger.isDebugEnabled();
    if(Level.ALL == level || Level.INFO == level)
      return logger.isInfoEnabled();
    if(Level.WARNING == level)
      return logger.isWarnEnabled();
    if(Level.SEVERE == level)
      return logger.isErrorEnabled();

    return false;

  }

}

